/*
 *
 *  Copyright (C) 1999 Laszlo Boloni
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "LookNFeelConfig.h"
#include "LookNFeelConfig.moc"
#include <kconfigbase.h>
#include <kconfig.h>
#include <qlayout.h>
#include <klocale.h>


LookNFeelConfig::LookNFeelConfig( QWidget *parent, const char* name )
    : KCModule (parent, name)
{
    layout = new QVBoxLayout(this);
    QGridLayout *glay = new QGridLayout(layout, 0, 0, 10);
    QLabel *label;
    int row=0;

    fontsize = new QSpinBox(5,50,1,this);
    connect(fontsize, SIGNAL(valueChanged(int)),
            this, SLOT(slotChanged()));
    glay->addWidget(fontsize, row, 2);
    label = new QLabel(fontsize, i18n("Base font size"), this);
    glay->addWidget(label, row, 1);
    row++;

    regular_bg = new KColorButton(this);
    connect(regular_bg, SIGNAL(changed(const QColor&)),
            this, SLOT(slotChanged()));
    glay->addWidget(regular_bg, row, 2);
    label = new QLabel(regular_bg, i18n("Color of a regular node"), this);
    glay->addWidget(label, row, 1);
    row++;

    category_bg = new KColorButton(this);
    connect(category_bg, SIGNAL(changed(const QColor&)),
            this, SLOT(slotChanged()));
    glay->addWidget(category_bg, row, 2);
    label = new QLabel(category_bg, i18n("Color of a category node"), this);
    glay->addWidget(label, row, 1);
    row++;

    positive_link = new KColorButton(this);
    connect(positive_link, SIGNAL(changed(const QColor&)),
            this, SLOT(slotChanged()));
    glay->addWidget(positive_link, row, 2);
    label = new QLabel(positive_link, i18n("Color of a positive link"), this);
    glay->addWidget(label, row, 1);
    row++;

    negative_link = new KColorButton(this);
    connect(negative_link, SIGNAL(changed(const QColor&)),
            this, SLOT(slotChanged()));
    glay->addWidget(negative_link, row, 2);
    label = new QLabel(negative_link, i18n("Color of a negative link"), this);
    glay->addWidget(label, row, 1);
    row++;

    transparent_bg = new QCheckBox(this);
    connect(transparent_bg, SIGNAL(clicked()), SLOT(slotChanged()));
    glay->addWidget(transparent_bg, row, 2);
    label = new QLabel(transparent_bg, i18n("Transparent background?"), this);
    glay->addWidget(label, row, 1);
    row++;

    animation = new QCheckBox(this);
    connect(animation, SIGNAL(clicked()), SLOT(slotChanged()));
    glay->addWidget(animation, row, 2);
    label = new QLabel(animation, i18n("Animation?"), this);
    glay->addWidget(label, row, 1);
    row++;

    askDelete = new QCheckBox(this);
    connect(askDelete, SIGNAL(clicked()), SLOT(slotChanged()));
    glay->addWidget(askDelete, row, 2);
    label = new QLabel(askDelete, i18n("Confirm deletes?"), this);
    glay->addWidget(label, row, 1);
    row++;

    glay->addColSpacing(0,15);
    glay->setColStretch(0,0);
    glay->setColStretch(1,5);
    glay->setColStretch(2,1);
    glay->setColStretch(3,3);

    layout->activate();
}

LookNFeelConfig::~LookNFeelConfig( ) {
}

void LookNFeelConfig::load() {
    config->setGroup("View");
    fontsize->setValue( config->readNumEntry("fontbase", 10));
    regular_bg->setColor(config->readColorEntry("regular_bg"));
    category_bg->setColor(config->readColorEntry("category_bg"));
    positive_link->setColor(config->readColorEntry("positive_link"));
    negative_link->setColor(config->readColorEntry("negative_link"));
    transparent_bg->setChecked(config->readBoolEntry("transparentBg"));
    animation->setChecked(config->readBoolEntry("animation"));
    config->setGroup("UserInterface");
    askDelete->setChecked(config->readBoolEntry("askDelete"));
}

void LookNFeelConfig::applySettings() {
    save();
}

void LookNFeelConfig::save() {
    config->setGroup("View");
    config->writeEntry("fontbase",fontsize->value());
    config->writeEntry("regular_bg", regular_bg->color());
    config->writeEntry("category_bg", category_bg->color());
    config->writeEntry("positive_link", positive_link->color());
    config->writeEntry("negative_link", negative_link->color());
    config->writeEntry("transparentBg", transparent_bg->isChecked());
    config->writeEntry("animation", animation->isChecked());
    config->setGroup("UserInterface");
    config->writeEntry("askDelete", askDelete->isChecked());
}

void LookNFeelConfig::defaults() {
}

void LookNFeelConfig::slotChanged() {
  emit changed(true);
}
