/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.


    gofaiDataSource.h

 */

#ifndef __GOFAI_DATASOURCE_H
#define __GOFAI_DATASOURCE_H

#include <qstring.h>
#include <qlist.h>
#include <qfile.h>
#include "gofaiItem.h"

class gofaiDataView;

/**
   The generic data source class
 */
class gofaiDataSource : public Domifiable {
    friend class gofaiDataView;
    friend class gofaiDataQuery;
 protected:
    gofaiDataSource();
 public:
    enum LinkDirection {From, To, Both};
    QString name; // the name of the data source
    QString creator; // the creator of the data source
    virtual ~gofaiDataSource() {};
 protected:
    bool changed;
    virtual void addItem(gofaiItem *it);
    virtual void removeItem(gofaiItem *it) = 0;
    virtual bool updateItem(gofaiItem *it) = 0;
    virtual void getAdjacentLinks(gofaiItem *it,
                                  LinkDirection direction,
                                  QList<gofaiItem> &list) = 0;
    // this is the one to be overwritten
    virtual void addItem2(gofaiItem *it) = 0;
    virtual gofaiItem* lookupItem(gofaiID){ return 0; };
    virtual gofaiItem* lookupItemByName(QString) { return 0; };
    virtual void find(QString, QList<gofaiItem> &) {};
  private:
    virtual gofaiID getUniqueId()=0;
};

#endif // __GOFAI_DATASOURCE_H
