/*
    This file is part of the KDE system
    Copyright (C)  1999,2000 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.


    gofaiDataSource_Memory.cpp

 */

#include <fstream.h>
#include <qdom.h>
#include <qbuffer.h>
#include <qtextstream.h>
#include <kmessagebox.h>
#include <kdebug.h>

#include <gofaiDataSource.h>
#include <gofaiDataView.h>
#include <gofaiConfig.h>
#include "gofaiDataSource_Memory.h"


/**
    gofaiDataSource_Memory
 */

#define INITIALIZE

gofaiDataSource_Memory::gofaiDataSource_Memory() :
    gofaiDataSource() {
    gofaiItem *it = it = new gofaiItem("Center", Node);
    // FIXME: what the heck is this
    addItem(it);
}

/**
   Creates a new id of the type "name_number".
   FIXME: to do efficient unique name scheme!
   Attention: this can be very inefficient, as it iterates
   through the whole thing.
 */
gofaiID gofaiDataSource_Memory::getUniqueId() {
    QString nextID;
    long jumpcount = 1;
 again:
    nextID.sprintf("%s_%ld",name.latin1(),db.count()+jumpcount);
    QListIterator<gofaiItem> iter(db);
    gofaiItem *it;
    // check by iteration if it is there
    for ( ; iter.current(); ++iter ) {
        it = iter.current();
        if (it->id == nextID) {
            jumpcount++;
            goto again;
        }
    }
    return nextID;
}

gofaiItem* gofaiDataSource_Memory::lookupItemByName(QString name) {
    gofaiItem *it;
    QListIterator<gofaiItem> iter(db);
    for ( ; iter.current(); ++iter ) {
        it = iter.current();
        if (it->name == name) {
            return it;
        }
    }
    return NULL;
};

void gofaiDataSource_Memory::find(QString prefix, QList<gofaiItem> &list) {
    gofaiItem *it;
    QListIterator<gofaiItem> iter(db); // iterator for employee list
    for ( ; iter.current(); ++iter ) {
        it = iter.current();
        if (it->name.find(prefix,0,false) != -1) {
            list.append(it);
        }
    }
};

gofaiItem* gofaiDataSource_Memory::lookupItem(gofaiID id) {
    if (id=="") {
        return db.at(0);
    }
    gofaiItem *it;
    QListIterator<gofaiItem> iter(db);
    for ( ; iter.current(); ++iter ) {
        it = iter.current();
        if (it->id == id) {
            return it;
        }
    }
    return NULL;
};




void gofaiDataSource_Memory::addItem2(gofaiItem *it) {
  changed = true;
  db.append(it);
};

void gofaiDataSource_Memory::removeItem(gofaiItem *it) {
  changed = true;
  if (it->type == Node) {
    // remove all the links to and from
    QListIterator<gofaiItem> iter(db);
    gofaiItem *it2;
    for ( ; iter.current(); ++iter ) {
      it2 = iter.current();
      // I am not sure this stuff did not break the iterator.
      if ((it2->from == it->id) || (it2->to == it->id)) {
        db.remove(it2);
      }
    }
  }
  db.remove(it);
};

bool gofaiDataSource_Memory::updateItem(gofaiItem */*it*/) {
    // we are in the memory, update is immediate
  changed = true;
  return true;
};


void gofaiDataSource_Memory::getAdjacentLinks(gofaiItem *it2,
                                  LinkDirection direction,
                                  QList<gofaiItem> &list) {

    QListIterator<gofaiItem> iter(db);
    gofaiItem *it;
    for ( ; iter.current(); ++iter ) {
        it = iter.current();
        if (it->type != Link) continue;
        switch(direction) {
            case From:
                if (it->from == it2->id) {
                    list.append(it);
                };
                break;
            case To:
                if (it->to == it2->id) {
                    list.append(it);
                };
                break;
            case Both:
                if ((it->from == it2->id) || (it->to == it2->id)) {
                    list.append(it);
                };
                break;
        }
    }
}


/**
   Writes out the content of the item into a branch of the
   DOM tree
 */
bool gofaiDataSource_Memory::writeToDOM(
          QDomDocument &doc, QDomElement &parent) {
    QDomElement items = doc.createElement("items");
    parent.appendChild(items);
    gofaiItem *it;
    for (it = db.first(); it != 0; it=db.next()) {
        it->writeToDOM(doc, items);
    }
    return true;
}

/**
   Reads the content of the item from a branch of
   the DOM tree
 */
bool gofaiDataSource_Memory::readFromDOM(QDomElement &root) {
    // start from scratch
    db.clear();
    QDomNode n = root.firstChild();
    while(!n.isNull()) {
        QDomElement e = n.toElement();
        gofaiItem *it = new gofaiItem("",Node);
        it->readFromDOM(e);
        db.append(it);
        n = n.nextSibling();
        // kdDebug() << "Element added" << endl;
    }
    return true;
}
