/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofaiItem.h

*/

#ifndef __GOFAI_ITEM_H
#define __GOFAI_ITEM_H

#include <qstring.h>
#include <qdatetime.h>
#include <qdom.h>
#include "Domifiable.h"


enum gofaiType {Node, Link};
typedef QString gofaiID;

class gofaiItem :
public Domifiable {
    static const int DefaultStrength = 100;
    static const int MaximumStrength = 10000;
    static const int MaxConfidence = 1000;
    static const int DefaultConfidence = 500;
    static const int MinConfidence = -1000;
//    static QString nextID;

public:
    gofaiItem(QString name, gofaiType type);
    virtual ~gofaiItem();

    void setLinkBetween(gofaiItem *one, gofaiItem *two);


    // Domifiable
    bool writeToDOM(QDomDocument &doc, QDomElement &parent);
    bool readFromDOM(QDomElement &e);
    /* these are the things which are saved and restored in the DB*/

    void setProperty(QString name, QString value);
    QString getProperty(QString name);

    gofaiID id;
    gofaiType type;
    QString name;
    QString description;
    QString link;
    QString owner;
    QString ontology;
    QString content_type;
    QString content_subtype;
    QString external_data;
    int strength;
    int confidence;
    bool category;
    QDateTime created, visited;
    gofaiID from, to;

private:
    QDict<QString> properties;
};

#endif // __GOFAI_ITEM_H
