/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofaiItemProcess.cpp - things to do with the item
 */

#include <fstream.h>
#include <stdio.h>
#include <kprocess.h>
#include <kapp.h>
#include "gofaiItemProcess.h"
#include <KonquerorIface_stub.h>
#include <kdebug.h>


DCOPRef gofaiItemProcess::theWindow;
bool gofaiItemProcess::foundKonq = false;


/**
   Browse to the link with the specified browser
 */
bool gofaiItemProcess::browseTo(gofaiItem *it) {
    enum BrowserType {Konqueror, Netscape, Mozilla, Opera, Other};
    // for the time being configurable later
    BrowserType browser = Konqueror;
    switch(browser) {
        case Konqueror:
            return konquerorBrowseTo(it);
        case Netscape:
            return netscapeBrowseTo(it);
        case Mozilla:
        case Opera:
        case Other:
            return false;
    }
    return false;
}


/**
   Find a running konqueror window
 */
void gofaiItemProcess::lookForKonqWindow(bool findLast = false) {
    KApplication *app = KApplication::kApplication();
    QCStringList apps = app->dcopClient()->registeredApplications();

    QValueList<DCOPRef> konqwindows;
    bool foundInstance = false;

    const char *lookForInstance;
    const char *lookForObject;
    if (!foundKonq) {
        lookForInstance = "konqueror";
        lookForObject = "konqueror-mainwindow";
    } else {
        lookForInstance = theWindow.app();
        lookForObject = theWindow.object();
    }

    for (QCStringList::ConstIterator it = apps.begin();
         it != apps.end();
         ++it) {
        if ((*it).left(strlen(lookForInstance)) == lookForInstance) {
            KonquerorIface_stub stub( (*it), "KonquerorIface" );
            konqwindows = stub.getWindows();
            foundInstance = true;
            if (!findLast) break;
        }
    }
    if (foundInstance == false) {
        foundKonq = false;
        return;
    }

    for(QValueList<DCOPRef>::Iterator it2 = konqwindows.begin();
        it2 != konqwindows.end();
        ++it2 ) {
        if ((*it2).object().left(strlen(lookForObject))==lookForObject) {
            foundKonq = true;
            theWindow = (*it2);
            if (!findLast) return;
        }
    }
    foundKonq = false;
}


/**
   Browse to the link using konqi
 */
bool gofaiItemProcess::konquerorBrowseTo(gofaiItem *item) {
    KApplication *app = KApplication::kApplication();
    QString theURL = addHTTP(item->link);
    // look for the existing instance
    lookForKonqWindow();
    // there is no running konqi instance, create a new one.
    if (!foundKonq) {
        app->invokeBrowser(theURL);
        lookForKonqWindow(true);
        // set it here: it will be the only one
        // in fact, we need the last one...
        return true;
    }
    // use the old one
    QByteArray data;
    QDataStream arg(data, IO_WriteOnly);
    arg << theURL;
    if (!app->dcopClient()->send(theWindow.app(),
                      theWindow.object(),
                      "openURL(QString)",
                      data)) {
        kdDebug() << "there was some error using DCOP.\n";
        return false;
    }
    return true;
}

/**
   Browse to the link using netscape
 */
bool gofaiItemProcess::netscapeBrowseTo(gofaiItem *it) {
    QString string = it->link;
    QString string2 = addHTTP(it->link);
    if (string2.isEmpty()) {
        return false;
    }

    char buffer[1024];
    sprintf(buffer, "\"openURL(%s)\"", string2.ascii());

    KProcess proc, ns_start;
    proc << "netscape";
    proc << "-remote";
    proc << buffer;
    proc << "-noraise";
    proc.start(KProcess::Block);
    if (proc.normalExit()) {
      cerr << "Normal exit of netscape command"
           << proc.exitStatus() << "\n";
      if (proc.exitStatus() != 0) {
        // probably netscape was not started, let's start it.
        ns_start << "netscape";
        ns_start << string2;
        ns_start.start(KProcess::DontCare);
        // now let
      }
    }
    return true;
};

/**
   Little utility function to add http:// in front of
   www
 */
QString gofaiItemProcess::addHTTP(QString link) {
    QString string = link.lower();
    if (string.left(4).compare("www.")==0) {
      string = "http://"+string;
      return string;
    }
    if (string.left(7).compare("http://")!=0) {
      return QString("");
    }
    return link;
};
