/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.


    gofaiVisualDataStruct.cpp

    gofai items in a visual way

*/

#include <fstream.h>
#include <stdlib.h>
#include <qcolor.h>
#include <qpainter.h>
#include <kdebug.h>
#include "gofaiVisualDataStruct.h"
#include "gofaiDataView.h"
#include "gofaiConfig.h"


gofaiVisualItem::gofaiVisualItem() {
    x = 0;
    y = 0;
    newx = 0;
    newy = 0;
    oldx = 0;
    oldy = 0;
    level = 0;
    newentry = true;
}

void gofaiVisualItem::drawIt(QPainter *p, gofaiDataView &dv) {
    if (type == NULL)
        kdError()<< "Problem!!!\n";
    type->drawIt(p, dv);
};

void gofaiVisualItem::moveTo(const QPoint &p) {
    oldx = x;
    oldy = y;
    newx = p.x();
    newy = p.y();
}

bool gofaiVisualItem::animateIt(QPainter *p, gofaiDataView &dv) {
    bool retval = true;
    int tolerance = 21;
    int speed = gofaiConfig::getInstance()->getAnimationSpeed();

    // jump directly for the introduced stuff
    if (x==0 && y==0) {
    // if (newentry) {
      setNewPosition();
      //  x=newx;
      // y=newy;
      retval = true;
      goto done;
    };
    //    if (gofaiConfig::getInstance()->animation) {
      retval = (x==newx && y==newy);
      if ( abs(x-newx) < tolerance) {
        x = newx;
      } else {
        x+= (newx-oldx) / speed;
        //retval = false;
      }
      if (abs(y-newy) < tolerance) {
        y = newy;
      } else {
        y +=  (newy-oldy) / speed;
      }
      /*
    } else {
      x = newx;
      y = newy;
      retval = true;
    }
      */
 done:
   drawIt(p, dv);
   return retval;
}

bool gofaiVisualItem::contains (const QPoint & p,
                                gofaiDataView &dv,
                                bool proper) const {
    return type->contains(p, dv, proper);
};

void gofaiVisualItem::setNewPosition() {
      x = newx;
      y = newy;
}

/**
   gofaiVisualItemType
 */
/*
gofaiVisualItemType::gofaiVisualItemType(gofaiVisualItem *vi) {
    this->vi = vi;
};
*/
/**
   gofaiVisualItemType_Rectange
 */

bool gofaiVisualItemType_Rectangle::contains (const QPoint & p,
                                              gofaiDataView &,
                                              bool proper) const {
    gofaiConfig *conf = gofaiConfig::getInstance();
    QFont font(conf->fontname, conf->getFontSize(vi->level));
    QFontMetrics fm(font);
    QString drawn_name = vi->val->name;
    QRect br = fm.boundingRect(drawn_name, -1);
    int space = 5;
    QRect rect(vi->x - space - br.width() / 2,
               vi->y - space - br.height(),
               br.width()+2 * space,
               br.height()+2 * space);
    return rect.contains(p, proper);
};


void gofaiVisualItemType_Rectangle::drawIt(QPainter *p, gofaiDataView &) {
    gofaiConfig *conf = gofaiConfig::getInstance();
    QFont font(conf->fontname, conf->getFontSize(vi->level));
    QFontMetrics fm(font);
    QString drawn_name = vi->val->name;
    QRect br = fm.boundingRect(drawn_name, -1);
    p->setFont(font);
    QColor tofill = vi->val->category ? conf->category_bg : conf->regular_bg;
    int space = 5;
    QPen pen;
    pen.setColor(QColor(0,0,0));
    pen.setWidth(1);
    p->setPen(pen);
    QRect rect(vi->x - space - br.width() / 2,
               vi->y - space - br.height(),
               br.width()+2 * space,
               br.height()+2 * space);
    if (vi->shadow) {
        rect.moveBy(5,5);
        p->fillRect(rect, QColor(0,0,0));
        rect.moveBy(-5, -5);
    }
    p->fillRect(rect, tofill);
    p->drawRect(rect);
    p->drawText(rect.left()+space, rect.bottom()-space, drawn_name);
};


/**
   gofaiVisualItemType_Link
 */
void gofaiVisualItemType_Link::drawIt(QPainter *p, gofaiDataView &dv) {
    gofaiConfig *conf = gofaiConfig::getInstance();
    gofaiVisualItem *vit1 = dv.containsItem(vi->val->from, true);
    gofaiVisualItem *vit2 = dv.containsItem(vi->val->to, true);
    // draw it only if both are in the data view
    if ((vit1 == NULL) || (vit2 == NULL)) {
        kdError() << "This link is in the view, but the endpoints are not!!!\n";
        /*
        cerr << "From:" << vi->val->from.name << "\n";
        cerr << "To:" << vi->val->to.name << "\n";
        */
        //exit(1);
        return;
    }
    if ( (vit1->x==0 && vit1->y==0) ||
         (vit2->x==0 && vit2->y==0)) {
      // meaningless link, don't draw it
      return;
    }
    QPen pen;
    // set the colors

    if (vi->val->confidence > 0) {
        pen.setColor(conf->positive_link);
    } else {
        pen.setColor(conf->negative_link);
    }
    pen.setWidth(3+ (abs(vi->val->confidence)-500)/100);
    // gray it out if it is not the current
    int portion = 2;
    if (vi->level != 0) {
        QColor a = pen.color();
        QColor bg = p->backgroundColor();
        QColor b( (a.red()+ (portion-1)*bg.red()) / portion,
                   (a.green()+(portion-1)*bg.green()) / portion,
                   (a.blue()+(portion-1)*bg.blue()) / portion);
        //pen.setStyle(DotLine);
        pen.setColor(b);
        pen.setWidth(1);
    }
    p->setPen(pen);
    p->drawLine(vit1->x, vit1->y, vit2->x, vit2->y);
};

/**
   Click on a link - this code is borrowed from Killustrator
   Copyright (C) 1998 Kai-Uwe Sattler (kus@iti.cs.uni-magdeburg.de)
 */
bool gofaiVisualItemType_Link::contains (const QPoint & p,
                                         gofaiDataView &dv,
                                         bool) const {
  gofaiVisualItem *vit1 = dv.containsItem(vi->val->from, true);
  gofaiVisualItem *vit2 = dv.containsItem(vi->val->to, true);
    // draw it only if both are in the data view
  if ((vit1 == NULL) || (vit2 == NULL)) {
      kdError() << "A link in dv, but the two edges are not in\n";
      if (vit1 != NULL) {
        kdError() << "From was: " << vit1->val->name << "\n";
      }
      if (vit2 != NULL) {
        kdError() << "To was: " << vit2->val->name << "\n";
      }
      return false;
  }

  float x1, x2, y1, y2, m, n, xp, yp, x, y;

  x = p.x();
  y = p.y();

  // x1 should be smaller than x2, otherwise swap
  if (vit1->x < vit2->x) {
    x1 = vit1->x;
    y1 = vit1->y;
    x2 = vit2->x;
    y2 = vit2->y;
  } else {
    x1 = vit2->x;
    y1 = vit2->y;
    x2 = vit1->x;
    y2 = vit1->y;
  }

  if (x1 - 3 <= x && x <= x2 + 3) {
    if (abs (int (x1 - x2)) < 5) {
      if ((y1 <= y  && y  <= y2) ||
          (y2 <= y  && y  <= y1)) {
        return true;
      }
    } else {
      // y = m * x + n;
      m = (y2 - y1) / (x2 - x1);
      n = y1 - m * x1;

      if (m > 1) {
        xp = (y - n) / m;
        if (xp - 5 <= x  && x <= xp + 5) {
          return true;
        }
      } else {
        yp = m * x + n;
        if (yp - 5 <= y && y <= yp + 5) {
          return true;
        }
      }
    }
  }
  return false;
};
