/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
/*
 * gofai_widget.h
 *
 */

#ifndef __GOFAI_WIDGET_H
#define __GOFAI_WIDGET_H

#include <qwidget.h>
#include <qtimer.h>
#include <qpopupmenu.h>
#include "gofaiDataView.h"
#include "gofaiDataSource.h"
#include "krootpixmap.h"


class gofaiWidget :
public QWidget {
  Q_OBJECT
public:
  gofaiWidget(QWidget *parent, gofaiDataView *dv);

  gofaiDataViewPointer currentNode;
  gofaiDataViewPointer currentLink;

  QPixmap bufferpixmap;
  QRect activezone;
  QRect contextzone;
  QPopupMenu *rmm_empty, *rmm_node, *rmm_link;

  void refresh();
protected:
  KRootPixmap *rootxpm;
  QTimer *animate_timer, *rwtimer;
  int rwcount;
  QPoint mousePressPoint;

  void setUpRMM();
  void drawIt( QPainter * );
  void paintEvent( QPaintEvent * );
  void mousePressEvent ( QMouseEvent * );
  void mouseReleaseEvent ( QMouseEvent * );
  void mouseDoubleClickEvent ( QMouseEvent * );
  void mouseMoveEvent ( QMouseEvent * );
  void resizeEvent ( QResizeEvent * );
  void moveEvent ( QMoveEvent * );
  void dragEnterEvent( QDragEnterEvent *);
  void dropEvent( QDropEvent *);
  QSize sizeHint() { return QSize(300,300); }
  void newNodeFromTextDrag(QString text);
public slots:
   void slotRepaint();
   void slotNewNode();
   void slotDeleteNode();
   void slotEditNode();
   void slotDeleteLink();
   void slotEditLink();
   void slotBrowseTo();
   void slotFind();
   void slotHistory();
   void slotRandomWalk();
   void slotRandomWalk2();
signals:
   void saveData(gofaiItem *it);
   void loadData(gofaiItem *it);
private:
    gofaiDataView *dv;
};

#endif // __GOFAI_WIDGET_H
