/*
    This file is part of the KDE system
    Copyright (C)  1999 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofai_common.c[[

    This is the common root of the standalone and embedded gofai

*/


#include "gofai_common.h"
#include "kdebug.h"
#include <qdom.h>
#include <qbuffer.h>
#include <qtextstream.h>


gofai_common::gofai_common() {
    // init
}

gofai_common::~gofai_common() {
    // deleting this stuff
}

void gofai_common::initialize(QWidget *parent) {
    // init gofai data source
    dv = new gofaiDataView();
    // visual elements
    splitter = new QSplitter(parent);
    splitter->setOrientation( QSplitter::Vertical );

    gofw = new gofaiWidget(splitter, dv);
    detail = new DetailWidget(splitter);

    // signals for synchronization with the detail widget
    gofw->connect(gofw, SIGNAL(saveData(gofaiItem*)),
            detail, SLOT(getValues(gofaiItem*)));
    gofw->connect(gofw, SIGNAL(loadData(gofaiItem*)),
            detail, SLOT(setValues(gofaiItem*)));

    dv->getView();
    dv->createVisuals(200,200,150);
    QValueList<int> splitinit;
    splitinit.append(80);
    splitinit.append(20);
    splitter->setSizes(splitinit);
}

/**
   Hmm: most probably this should go into the dataview!!!
 */
bool gofai_common::loadFile(const QString & filename) {
  QFile f(filename);
  if (!f.open(IO_ReadOnly))
    return false;
  QDomDocument *doc = new QDomDocument();
  if (!doc->setContent(&f)) {
    // this is probably a parsing error
    return false;
  };
  if ( doc->doctype().name() != "goph" ) {
    return false;
  }
  // get the document
  QDomElement goph = doc->documentElement();
  QDomNode n = goph.firstChild();
  QDomElement e = n.toElement();
  if (!dv->readFromDOM(e)) {
      kdDebug() << "Error while reading the dataview\n";
      return false;
  }
  return true;
}

/**
   Hmm: most probably this should go into the dataview!!!
 */
bool gofai_common::saveToFile(const QString &destination) {
    // Guarantee that we save the current changes
    detail->getValues(dv->center->val); /// ???
    QDomDocument doc( "goph" );
    doc.appendChild( doc.createProcessingInstruction(
        "xml", "version=\"1.0\" encoding=\"UTF-8\"" ) );
    QDomElement goph = doc.createElement( "goph" );
    goph.setAttribute( "author", "Laszlo Boloni" );
    goph.setAttribute( "email", "boloni@excite.com" );
    goph.setAttribute( "editor", "gofai" );
    goph.setAttribute( "mime", "application/x-gofai" );
    goph.setAttribute( "version", "1.0" );

    doc.appendChild(goph);
    if (!dv->writeToDOM(doc, goph)) {
        return false;
    }
    QFile file(destination);
    if (!file.open(IO_WriteOnly)) {
        return false;
    }
    QTextStream str(&file);
    str << doc;
    file.close();
    return true;
}
