/*
    This file is part of the KDE system
    Copyright (C)  1999,2000 Boloni Laszlo

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

    gofai.cpp - Good Old Fashioned Artificial Intelligence With a Modern Twist
 */

#include "gofai_part.h"

#include <kinstance.h>
#include <klocale.h>
#include <kaboutdata.h>
#include <kaction.h>
#include <kparts/partmanager.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <qlabel.h>
#include <qsplitter.h>
#include <qdom.h>

#include "gofai_common.h"

extern "C"
{
  /**
   * This function is the 'main' function of this part.  It takes
   * the form 'void *init_lib<library name>()  It always returns a
   * new factory object
   */
  void *init_libgofai()  {
    return new gofaiFactory;
  }
};

/**
 * We need one static instance of the factory for our C 'main'
 * function
 */
KInstance *gofaiFactory::s_instance = 0L;

gofaiFactory::gofaiFactory()
{
}

gofaiFactory::~gofaiFactory()
{
  if (s_instance)
    delete s_instance;

  s_instance = 0;
}

QObject *gofaiFactory::create(QObject *parent, const char *name, const char*,
                               const QStringList& )
{
  QObject *obj = new gofaiPart((QWidget*)parent, name);
  emit objectCreated(obj);
  return obj;
}

KInstance *gofaiFactory::instance()
{
  if ( !s_instance )
    {
      //KAboutData *about("gofai", I18N_NOOP("Gofai"), "1.0");
      s_instance = new KInstance("gofai");
    }
  return s_instance;
}

/**
    And here comes the part itself

 */
gofaiPart::gofaiPart(QWidget *parent, const char *name)
  : KParts::ReadOnlyPart(parent, name)
{
  setInstance(gofaiFactory::instance());
  m_extension = new gofaiBrowserExtension(this);
  initialize(parent);
  setXMLFile("gofai_part.rc");
}

gofaiPart::~gofaiPart()
{
  closeURL();
}

bool gofaiPart::openFile() {
  return gofai_common::loadFile(m_file);
}

/**
   Browser extension stuff
 */
gofaiBrowserExtension::gofaiBrowserExtension(gofaiPart *parent)
  : KParts::BrowserExtension(parent, "gofaiBrowserExtension")
{
}

gofaiBrowserExtension::~gofaiBrowserExtension()
{
}
#include "gofai_part.moc"
